@@ -7,9 +7,8 @@ The core classes and can be created and saved as shown:
 <div markdown="1" style="width: max-content;">
 |                  | Create ✨                                                                                                                                             | Open/Load 📂                                                                                                                                           | Save 💾                                                                       | Download 📥                                                                                       | Upload 📤                                                                         |
 |------------------|-------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------|-------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------|------------------------------------------------------------------------------------|
-| **`Dataset`**    | [`Dataset(…)`](webknossos/dataset/dataset.html#webknossos.dataset.dataset.Dataset),<br/>[`Dataset.from_images(…)`](webknossos/dataset/dataset.html#webknossos.dataset.dataset.Dataset.from_images) | [`Dataset.open(…)`](webknossos/dataset/dataset.html#webknossos.dataset.dataset.Dataset.open),<br/>[`Dataset.open_remote(…)`](webknossos/dataset/dataset.html#webknossos.dataset.dataset.Dataset.open_remote) | *N/A*[^ds-save]                                                               | [`Dataset.download(…)`](webknossos/dataset/dataset.html#webknossos.dataset.dataset.Dataset.download)                         | [`dataset.upload(…)`](webknossos/dataset/dataset.html#webknossos.dataset.dataset.Dataset.upload)              |
-| **`Annotation`** | [`Annotation(…)`](webknossos/annotation/annotation.html#webknossos.dataset.dataset.Annotation)                                                                          | [`Annotation.load(…)`](webknossos/annotation/annotation.html#Annotation.load)                                                                          | [`annotation.save(…)`](webknossos/annotation/annotation.html#webknossos.dataset.dataset.Annotation.save) | [`Annotation.download(…)`](webknossos/annotation/annotation.html#Annotation.webknossos.dataset.dataset.download)[^anno-down] | [`annotation.upload(…)`](webknossos/annotation/annotation.html#webknossos.dataset.dataset.Annotation.upload)  |
-| **`Skeleton`**   | [`Skeleton(…)`](webknossos/skeleton/skeleton.html#webknossos.dataset.dataset.webknossos.dataset.dataset.Skeleton)                                                                                  | [`Skeleton.load(…)`](webknossos/skeleton/skeleton.html#webknossos.dataset.dataset.Skeleton.load)                                                                                  | [`skeleton.save(…)`](webknossos/skeleton/skeleton.html#webknossos.dataset.dataset.Skeleton.save)         | *only in an annotation*                                                                           | *only in an annotation*                                                            |
+| **`Dataset`**    | [`Dataset(…)`](webknossos/dataset/dataset.html#webknossos.dataset.Dataset),<br/>[`Dataset.from_images(…)`](webknossos/dataset/dataset.html#webknossos.dataset.Dataset.from_images) | [`Dataset.open(…)`](webknossos/dataset/dataset.html#webknossos.dataset.Dataset.open),<br/>[`RemoteDataset.open(…)`](webknossos/dataset/remote_dataset.html#webknossos.dataset.RemoteDataset.open) | *N/A*[^ds-save]                                                               | [`RemoteDataset.download(…)`](webknossos/dataset/remote_dataset.html#webknossos.dataset.RemoteDataset.download)                         | [`dataset.upload(…)`](webknossos/dataset/dataset.html#webknossos.dataset.Dataset.upload)              |
+| **`Annotation`** | [`Annotation(…)`](webknossos/annotation/annotation.html#webknossos.annotation.Annotation)                                                                          | [`Annotation.load(…)`](webknossos/annotation/annotation.html#webknossos.annotation.Annotation.load)                                                                          | [`annotation.save(…)`](webknossos/annotation/annotation.html#webknossos.annotation.Annotation.save) | [`Annotation.download(…)`](webknossos/annotation/annotation.html#webknossos.annotation.Annotation.download)[^anno-down] | [`annotation.upload(…)`](webknossos/annotation/annotation.html#webknossos.annotation.Annotation.upload)  |
 </div>
 </div>
 
@@ -18,7 +17,7 @@ The core classes and can be created and saved as shown:
 
 Additionally, we provide the various geometrical primitives, e.g. `Vec3Int`, `BoundingBox` and `Mag`.
 
-The `User`, `Project` and `Task` classes provide WEBKNOSSOS server interactions for administration purposes.
+The `User`, `Team`, `Project` and `Task` classes provide WEBKNOSSOS server interactions for administration purposes.
 Server interactions may require [authentication](webknossos/client/context.html) e.g. via `webknossos_context`.
 """
 
@@ -28,6 +27,7 @@ from .administration import *
 from .annotation import *
 from .client import *
 from .dataset import *
+from .dataset_properties import *
 from .datastore import *
 from .geometry import *
 from .skeleton import *
@@ -45,3 +45,7 @@ if not current_version == "0.0.0":
         os.environ["WEBKNOSSOS_SKIP_VERSION_CHECK"] = "True"
         # Schedule the version check to run non-blocking in a background thread
         check_version_in_background(current_version)
+import os
+
+# This needs to be set to make sure the encoding is not chunked when uploading
+os.environ["AWS_REQUEST_CHECKSUM_CALCULATION"] = "WHEN_REQUIRED"
