@@ -1,8 +1,10 @@
+# ruff: noqa: F401 imported but unused
 from .branchpoint import Branchpoint
 from .comment import Comment
 from .edge import Edge
 from .group import Group
 from .meta import Meta
+from .metadata_entry import MetadataEntry
 from .nml import Nml
 from .node import Node
 from .parameters import Parameters
