@@ -1,4 +1,5 @@
 # ruff: noqa: F401 imported but unused
 from .project import Project
 from .task import Task, TaskType
-from .user import Team, User
+from .team import Team
+from .user import User
