@@ -1,10 +1,9 @@
 from collections.abc import Iterator
 from contextlib import contextmanager
-from os import PathLike
-from pathlib import Path
 
 import numpy as np
 from pims import FramesSequenceND
+from upath import UPath
 
 try:
     from pylibCZIrw import czi as pyczi
@@ -38,8 +37,8 @@ class PimsCziReader(FramesSequenceND):
     # See http://soft-matter.github.io/pims/v0.6.1/custom_readers.html#plugging-into-pims-s-open-function
     class_priority = 20
 
-    def __init__(self, path: PathLike, czi_channel: int = 0) -> None:
-        self.path = Path(path)
+    def __init__(self, path: UPath, czi_channel: int = 0) -> None:
+        self.path = UPath(path)
         self.czi_channel = czi_channel
         self.axis_offsets: dict[str, int] = {}
         super().__init__()
