@@ -23,9 +23,10 @@ class RemoteDatasetRegistry(LazyReadOnlyDict[str, "RemoteDataset"]):
         tags: str | Sequence[str] | None,
         folder_id: str | None,
     ) -> None:
+        from webknossos.dataset.remote_dataset import RemoteDataset
+
         from ..administration.user import User
         from ..client.context import _get_context
-        from .dataset import Dataset
 
         context = _get_context()
         client = context.api_client_with_auth
@@ -54,7 +55,7 @@ class RemoteDatasetRegistry(LazyReadOnlyDict[str, "RemoteDataset"]):
         super().__init__(
             entries=dict(zip(datasets_ids, datasets_ids)),
             func=webknossos_context(context.url, context.token)(
-                lambda dataset_id: Dataset.open_remote(dataset_id=dataset_id),
+                lambda dataset_id: RemoteDataset.open(dataset_id=dataset_id),
             ),
         )
 
