@@ -3,7 +3,7 @@ import calendar
 import functools
 import json
 import logging
-import sys
+import os
 import time
 import warnings
 from collections.abc import Callable, Iterable, Iterator, Mapping, Sequence
@@ -14,12 +14,12 @@ from inspect import getframeinfo, stack
 from multiprocessing import cpu_count
 from multiprocessing.pool import ThreadPool
 from os import PathLike
-from pathlib import Path, PosixPath, WindowsPath
 from shutil import copyfileobj, move
 from threading import Thread
 from typing import (
     Any,
     Protocol,
+    TypeGuard,
     TypeVar,
 )
 from urllib.parse import urlparse
@@ -31,8 +31,7 @@ from cluster_tools import Executor, get_executor
 from packaging.version import InvalidVersion, Version
 from rich.progress import Progress
 from upath import UPath
-
-from .dataset.defaults import DEFAULT_BACKOFF_FACTOR, DEFAULT_NUM_RETRIES
+from upath.implementations.local import PosixUPath, WindowsUPath
 
 logger = logging.getLogger(__name__)
 
@@ -43,11 +42,28 @@ ReturnType = TypeVar("ReturnType")
 
 def _is_exception_retryable(exception: Exception) -> bool:
     exception_str_lower = str(exception).lower()
-    if "too many requests" in exception_str_lower or "gateway" in exception_str_lower:
+    if (
+        "too many requests" in exception_str_lower
+        or "gateway" in exception_str_lower
+        or "unavailable" in exception_str_lower
+        or "unknown error" in exception_str_lower
+    ):
         return True
     return False
 
 
+DEFAULT_NUM_RETRIES = (
+    int(os.environ["DEFAULT_NUM_RETRIES"])
+    if "DEFAULT_NUM_RETRIES" in os.environ
+    else 20
+)
+DEFAULT_BACKOFF_FACTOR = (
+    float(os.environ["DEFAULT_BACKOFF_FACTOR"])
+    if "DEFAULT_BACKOFF_FACTOR" in os.environ
+    else 1.75
+)
+
+
 def call_with_retries(
     fn: Callable[[], ReturnType],
     num_retries: int = DEFAULT_NUM_RETRIES,
@@ -234,31 +250,6 @@ def setup_warnings() -> None:
     warnings.filterwarnings("default", category=DeprecationWarning, module="webknossos")
 
 
-def setup_logging(args: argparse.Namespace) -> None:
-    log_path = Path(f"./logs/cuber_{time.strftime('%Y-%m-%d_%H%M%S')}.txt")
-
-    console_log_level = logging.DEBUG if args.verbose else logging.INFO
-    file_log_level = logging.DEBUG
-
-    logging_formatter = logging.Formatter("%(asctime)s %(levelname)s %(message)s")
-
-    # Always set the global log level to the more verbose of console_log_level and
-    # file_log_level to allow to log with different log levels to console and files.
-    root_logger = logging.getLogger()
-    root_logger.setLevel(min(console_log_level, file_log_level))
-
-    console = logging.StreamHandler(sys.stdout)
-    console.setLevel(console_log_level)
-    console.setFormatter(logging_formatter)
-    root_logger.addHandler(console)
-
-    log_path.parent.mkdir(parents=True, exist_ok=True)
-    file_handler = logging.FileHandler(log_path, mode="w", encoding="UTF-8")
-    file_handler.setLevel(file_log_level)
-    file_handler.setFormatter(logging_formatter)
-    root_logger.addHandler(file_handler)
-
-
 def add_verbose_flag(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
         "--verbose", help="Verbose output", dest="verbose", action="store_true"
@@ -291,22 +282,22 @@ def count_defined_values(values: Iterable[Any | None]) -> int:
     return sum(i is not None for i in values)
 
 
-def is_fs_path(path: UPath) -> bool:
-    from upath.implementations.local import PosixUPath, WindowsUPath
-
-    return not isinstance(path, UPath) or isinstance(
-        path, PosixPath | WindowsPath | PosixUPath | WindowsUPath
-    )
+def is_fs_path(
+    path: UPath,
+) -> TypeGuard[PosixUPath | WindowsUPath]:
+    return isinstance(path, PosixUPath | WindowsUPath)
 
 
 def is_remote_path(path: UPath) -> bool:
-    return not is_fs_path(path)
+    return isinstance(path, UPath) and not is_fs_path(path)
 
 
-def resolve_if_fs_path(path: UPath) -> UPath:
-    if is_fs_path(path):
-        return path.resolve()
-    return path
+def cheap_resolve(path: UPath) -> UPath:
+    from upath.implementations.http import HTTPPath
+
+    if isinstance(path, HTTPPath):
+        return path.resolve(follow_redirects=False, strict=False)
+    return path.resolve()
 
 
 def is_writable_path(path: UPath) -> bool:
@@ -351,11 +342,11 @@ def copytree(
     threads: int | None = 10,
     progress_desc: str | None = None,
 ) -> None:
-    def _walk(path: UPath, base_path: UPath) -> Iterator[tuple[Path, tuple[str, ...]]]:
+    def _walk(path: UPath, base_path: UPath) -> Iterator[tuple[UPath, tuple[str, ...]]]:
         # base_path.parts is a prefix of path.parts; strip it
         assert len(path.parts) >= len(base_path.parts)
         assert path.parts[: len(base_path.parts)] == base_path.parts
-        yield (path, path.parts[len(base_path.parts) :])
+        yield (path, tuple(path.parts[len(base_path.parts) :]))
 
         if path.is_dir():
             for p in path.iterdir():
@@ -368,6 +359,7 @@ def copytree(
 
     def _copy(args: tuple[UPath, UPath, tuple[str, ...]]) -> None:
         in_path, out_path, sub_path = args
+
         with (
             _append(in_path, sub_path).open("rb") as in_file,
             _append(out_path, sub_path).open("wb") as out_file,
@@ -394,6 +386,10 @@ def copytree(
 
 
 def movetree(in_path: UPath, out_path: UPath) -> None:
+    assert is_fs_path(in_path), f"Move is only supported for local paths, got {in_path}"
+    assert is_fs_path(out_path), (
+        f"Move is only supported for local paths, got {out_path}"
+    )
     move(in_path, out_path)
 
 
@@ -523,15 +519,17 @@ def safe_is_relative_to(path: UPath, base_path: UPath) -> bool:
     return False
 
 
-def enrich_path(path: str | PathLike | UPath, dataset_path: UPath) -> UPath:
+def enrich_path(
+    path: str | PathLike | UPath, dataset_path: UPath | None = None
+) -> UPath:
     upath = UPath(path)
     if upath.protocol in ("http", "https"):
         from .client.context import _get_context
-        from .dataset.defaults import SSL_CONTEXT
+        from .ssl_context import SSL_CONTEXT
 
         # To setup the mag for non-public remote paths, we need to get the token from the context
         wk_context = _get_context()
-        token = wk_context.datastore_token
+        token = wk_context.token
         return UPath(
             path,
             headers={} if token is None else {"X-Auth-Token": token},
@@ -539,37 +537,46 @@ def enrich_path(path: str | PathLike | UPath, dataset_path: UPath) -> UPath:
         )
 
     elif upath.protocol == "s3":
-        if (
-            upath.storage_options.get("client_kwargs", {}).get("endpoint_url")
-            is not None
-        ):
+        if upath.storage_options.get("endpoint_url") is not None:
             return upath
         parsed_url = urlparse(str(upath))
         endpoint_url = f"https://{parsed_url.netloc}"
         bucket, key = parsed_url.path.lstrip("/").split("/", maxsplit=1)
 
-        return UPath(
-            f"s3://{bucket}/{key}", client_kwargs={"endpoint_url": endpoint_url}
-        )
+        return UPath(f"s3://{bucket}/{key}", endpoint_url=endpoint_url)
 
     if not upath.is_absolute():
-        return resolve_if_fs_path(dataset_path / upath)
-    return resolve_if_fs_path(upath)
+        assert dataset_path is not None, (
+            f"dataset_path must be set if {path} is not absolute"
+        )
+        return cheap_resolve(dataset_path / upath.as_posix())
+    return cheap_resolve(upath)
+
+
+def _ensure_trailing_slash(path: str) -> str:
+    if not path.endswith("/"):
+        return path + "/"
+    return path
 
 
 def dump_path(path: UPath, dataset_path: UPath | None) -> str:
+    if dataset_path is not None:
+        if not dataset_path.is_absolute():
+            raise ValueError("dataset_path must be an absolute path.")
+        dataset_path = cheap_resolve(dataset_path)
     if is_fs_path(path) and not path.is_absolute():
         if dataset_path is None:
             raise ValueError("dataset_path must be provided when path is not absolute.")
         path = dataset_path / path
-    path = resolve_if_fs_path(path)
+    path = cheap_resolve(path)
     if dataset_path is not None:
-        if path.as_posix().startswith(dataset_path.as_posix()):
-            return "./" + path.as_posix().removeprefix(dataset_path.as_posix()).lstrip(
-                "/"
-            )
+        path_str = path.as_posix()
+        dataset_path_str = _ensure_trailing_slash(dataset_path.as_posix())
+        if path_str.startswith(dataset_path_str):
+            return "./" + path_str.removeprefix(dataset_path_str)
         if safe_is_relative_to(path, dataset_path):
             return "./" + path.relative_to(dataset_path).as_posix()
     if path.protocol == "s3":
-        return f"s3://{urlparse(path.storage_options['client_kwargs']['endpoint_url']).netloc}/{path.path}"
+        endpoint_url = path.storage_options["endpoint_url"]
+        return f"s3://{urlparse(endpoint_url).netloc}/{path.path}"
     return path.as_posix()
